<?php

namespace Drupal\synpay\Plugin\Synpay;

use Drupal\synpay\PluginManager\SynpayPluginBase;
use Drupal\synpay\PluginManager\SynpayPluginInterface;
use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\Component\Serialization\Json;

/**
 * Provides Tinkoff Gateway.
 *
 * @SynpayAnnotation(
 *   id = "tinkoff_qr",
 *   title = @Translation("Tinkoff Qr"),
 * )
 */
class SynpayTinkoffQr extends SynpayTinkoff implements SynpayPluginInterface {

  /**
   * {@inheritdoc}
   */
  public function __construct(array $configuration, $plugin_id, $plugin_definition) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
  }

  /**
   * {@inheritdoc}
   */
  public function registerOrder($order_id, PaymentInterface $payment, $params) {
    if (is_null($this->client)) {
      return FALSE;
    }
    $order = $payment->getOrder();
    $amount = $payment->getAmount()->getNumber();
    $order_amount = self::PRECISION ? $amount : bcmul($amount, 100, 0);
    $terminal_key = trim($this->user);
    $data = [
      'headers' => $this->headers,
      'TerminalKey' => $terminal_key,
      'Amount' => $order_amount,
      'OrderId' => $order_id,
      'Language' => $this->lang,
      'NotificationURL' => "{$this->host}/payment/notify/{$payment->getPaymentGatewayId()}",
      'SuccessURL' => $params['returnUrl'],
      'FailURL' => $params['failUrl'],
      'Receipt' => [
        'Email' => $order->getEmail(),
        'Taxation' => $this->config->get("{$this->plugin_id}_taxation"),
        'Items' => $this->getItems($order),
      ],
    ];
    try {
      $answer = $this->client->post("{$this->api}Init", ['json' => $data]);
      $code = $answer->getStatusCode();
      if ($code == 200) {
        $body = $answer->getBody()->getContents();
        $response = JSON::decode($body);
        if ($response['Success']) {
          return $this->getQr($terminal_key, $response['PaymentId']);
          return [
            'orderId' => $response['PaymentId'],
            'formUrl' => $response['PaymentURL'],
          ];
        }
        else {
          \Drupal::messenger()->addError($response['Message']);
          \Drupal::messenger()->addError($response['Details']);
          return FALSE;
        }
      }
    }
    catch (ActionException $exception) {
      $message = $exception->getMessage();
      \Drupal::messenger()->addError($message);
      \Drupal::messenger()->addWarning(t('Check login/password to your payment gateway!'));
    }
    return FALSE;
  }

  /**
   * Получить Qr.
   */
  private function getQr(string $terminal_key, string $payment_id) {
    $token = hash('sha256', $terminal_key . $payment_id);
    $data = [
      'TerminalKey' => $terminal_key,
      'PaymentId' => $payment_id,
      'Token' => $token,
    ];
    $answer = $this->client->post("{$this->api}GetQr", ['json' => $data]);
    $code = $answer->getStatusCode();
    if ($code == 200) {
      $body = $answer->getBody()->getContents();
      $response = JSON::decode($body);
      if ($response['Success']) {
        return [
          'orderId' => $response['PaymentId'],
          'formUrl' => $response['Data'],
        ];
      }
      else {
        \Drupal::messenger()->addError($response['Message']);
        \Drupal::messenger()->addError($response['Details']);
        return FALSE;
      }
    }
  }

  /**
  * Setting Form.
  */
  public function settingForm() {
    $config = \Drupal::config('synpay.settings');
    $form["{$this->plugin_id}_test_login"] = [
      '#type' => 'textfield',
      '#title' => "Тестовый логин $this->title",
      '#maxlength' => 100,
      '#size' => 100,
      '#default_value' => $config->get("{$this->plugin_id}_test_login"),
    ];
    $form["{$this->plugin_id}_test_token"] = [
      '#type' => 'textfield',
      '#title' => "Тестовый токен $this->title",
      '#maxlength' => 100,
      '#size' => 100,
      '#default_value' => $config->get("{$this->plugin_id}_test_token"),
    ];
    $form["{$this->plugin_id}_live_login"] = [
      '#type' => 'textfield',
      '#title' => "Рабочий логин $this->title",
      '#maxlength' => 100,
      '#size' => 100,
      '#default_value' => $config->get("{$this->plugin_id}_live_login"),
    ];
    $form["{$this->plugin_id}_live_token"] = [
      '#type' => 'textfield',
      '#title' => "Рабочий токен $this->title",
      '#maxlength' => 100,
      '#size' => 100,
      '#default_value' => $config->get("{$this->plugin_id}_live_token"),
    ];
    // $form["{$this->plugin_id}_send_receipt"] = [
    //   '#type' => 'select',
    //   '#title' => 'Передавать данные для формирования чека',
    //   '#options' => [
    //       'no' => 'Нет',
    //       'yes' => 'Да'
    //   ],
    //   '#default_value' => $config->get("{$this->plugin_id}_send_receipt"),
    // ];
    $form["{$this->plugin_id}_taxation"] = [
      '#type' => 'select',
      '#title' => 'Система налогообложения',
      '#options' => [
        'osn' => 'Общая',
        'usn_income' => 'Упрощенная (доходы)',
        'usn_income_outcome' => 'Упрощенная (доходы минус расходы)',
        'patent' => 'Патентная',
        'envd' => 'Единый налог на вмененный доход',
        'esn' => 'Единый сельскохозяйственный налог',
      ],
      '#default_value' => $config->get("{$this->plugin_id}_taxation"),
    ];
    $form["{$this->plugin_id}_tax"] = [
      '#type' => 'select',
      '#title' => 'Ставка НДС',
      '#options' => [
        'none' => 'без НДС',
        'vat0' => '0%',
        'vat10' => '10%',
        'vat20' => '20%',
        'vat110' => '10/110',
        'vat120' => '20/120',
      ],
      '#default_value' => $config->get("{$this->plugin_id}_tax"),
    ];
    return $form;
  }

  /**
   * Log.
   *
   * @param string $message
   *   Log message.
   */
  private function log($message) {
    \Drupal::logger('SynpayTinkoff')->info($message);
  }

}
